@@ -155,6 +155,7 @@ gtk_html_a11y_get_type (void)
 static void
 gtk_html_a11y_finalize (GObject *obj)
 {
+        G_OBJECT_CLASS (parent_class)->finalize (obj);
 }
 
 static void
@@ -309,7 +310,9 @@ gtk_html_a11y_grab_focus_cb(GtkWidget * widget)
 
 
 	focus_object = gtk_html_a11y_get_focus_object (widget);
-	g_return_if_fail (focus_object != NULL);
+	if (focus_object == NULL)
+		return;
+
 	obj = gtk_widget_get_accessible (widget);
 	g_object_set_data (G_OBJECT(obj), "gail-focus-object", focus_object);
 
@@ -388,7 +391,8 @@ gtk_html_a11y_delete_object_cb (GtkWidget * widget, int pos, int len)
 	}
 
 	if (G_IS_HTML_A11Y_TEXT(a11y)) {
-		g_signal_emit_by_name (a11y, "text_changed::delete", cursor->offset, len);
+		g_signal_emit_by_name (a11y, "text_changed::delete", pos, len);
+		printf ("the cur is %d\n", cursor->offset);
 	}
 }
 
@@ -406,7 +410,7 @@ gtk_html_a11y_new (GtkWidget *widget)
 	atk_object_initialize (accessible, widget);
 
 	accessible->role = ATK_ROLE_PANEL;
-	g_signal_connect_after (widget, "grab_focus", 
+	g_signal_connect (widget, "grab_focus", 
 			G_CALLBACK (gtk_html_a11y_grab_focus_cb),
 			NULL);
 	g_signal_connect (widget, "cursor_changed",
@@ -419,7 +423,8 @@ gtk_html_a11y_new (GtkWidget *widget)
 			G_CALLBACK(gtk_html_a11y_delete_object_cb),
 			NULL);
 
-	html_utils_get_accessible(GTK_HTML(widget)->engine->clue, accessible);
+	if (GTK_HTML(widget)->engine->clue)
+		html_utils_get_accessible(GTK_HTML(widget)->engine->clue, accessible);
 
 	/* printf ("created new gtkhtml accessible object\n"); */
 
