@@ -119,7 +119,8 @@ destroy (HTMLObject *o)
 			g_assert (element->widget->parent == element->parent);
 			gtk_container_remove (GTK_CONTAINER (element->parent), element->widget);
 		} else {
-		       	gtk_object_sink (GTK_OBJECT (element->widget));
+			g_object_ref_sink (element->widget);
+			g_object_unref (element->widget);
 		}
 	}
 
@@ -161,7 +162,7 @@ html_embedded_real_calc_size (HTMLObject *self, HTMLPainter *painter, GList **ch
 	GtkWidget *widget;
 	HTMLEmbedded *emb = HTML_EMBEDDED (self);
 	gint pixel_size;
-	gint old_width, old_ascent, old_descent;
+	gint old_width, old_ascent;
 	GtkRequisition requisition;
 
 	widget = emb->widget;
@@ -172,7 +173,6 @@ html_embedded_real_calc_size (HTMLObject *self, HTMLPainter *painter, GList **ch
 
 	old_width = self->width;
 	old_ascent = self->ascent;
-	old_descent = self->descent;
 
 	requisition.width = requisition.height = 0;
 	gtk_widget_size_request (widget, &requisition);
@@ -351,7 +351,7 @@ html_embedded_object_changed (GtkHTMLEmbedded *eb, HTMLEngine *e)
 
 	object = HTML_OBJECT (g_object_get_data (G_OBJECT (eb), "embeddedelement"));
 	if (object)
-		html_object_calc_size (object, e->painter, FALSE);
+		html_object_calc_size (object, e->painter, NULL);
 	
 	html_engine_schedule_update(e);
 }
