@@ -35,6 +35,7 @@
 #include "htmltable.h"
 #include "htmltablecell.h"
 #include "htmltablepriv.h"
+#include "htmlselection.h"
 #include "htmlundo.h"
 
 HTMLTable *
@@ -188,6 +189,7 @@ html_engine_table_goto_col (HTMLEngine *e, HTMLTable *table, gint col)
 	HTMLTableCell *cell;
 
 	if (html_engine_goto_table_0 (e, table)) {
+		html_cursor_forward (e->cursor, e);
 		cell = html_engine_get_table_cell (e);
 		while (cell && cell->col != col && HTML_OBJECT (cell)->parent == HTML_OBJECT (table)) {
 			html_engine_next_cell (e, FALSE);
@@ -206,6 +208,7 @@ html_engine_table_goto_row (HTMLEngine *e, HTMLTable *table, gint row)
 	HTMLTableCell *cell;
 
 	if (html_engine_goto_table_0 (e, table)) {
+		html_cursor_forward (e->cursor, e);
 		cell = html_engine_get_table_cell (e);
 		while (cell && cell->row != row && HTML_OBJECT (cell)->parent == HTML_OBJECT (table)) {
 			html_engine_next_cell (e, FALSE);
@@ -313,6 +316,7 @@ delete_cells_undo_destroy (HTMLUndoData *undo_data)
 
 	for (i = 0; i < data->size; i ++)
 		html_object_destroy (HTML_OBJECT (data->cells [i]));
+	g_free(data->cells);
 }
 
 static DeleteCellsUndo *
@@ -339,18 +343,23 @@ delete_column_undo_action (HTMLEngine *e, HTMLUndoData *undo_data, HTMLUndoDirec
 	HTMLTable *table;
 
 	table = html_engine_get_table (e);
+	if (!table) {
+		html_cursor_jump_to_position (e->cursor, e, position_after + 1);
+		table = html_engine_get_table (e);
+	}
+	g_assert (table);
 	g_assert (data->size == table->totalRows);
 	html_table_insert_column (table, e, data->pos, data->cells, html_undo_direction_reverse (dir));
 }
 
 static void
-delete_column_setup_undo (HTMLEngine *e, HTMLTableCell **column, gint size, guint position_before, gint col, HTMLUndoDirection dir)
+delete_column_setup_undo (HTMLEngine *e, HTMLTableCell **column, gint size, guint position_after, gint col, HTMLUndoDirection dir)
 {
 	html_undo_add_action (e->undo,
 			      html_undo_action_new ("Delete table column", delete_column_undo_action,
 						    HTML_UNDO_DATA (delete_cells_undo_new (column, size, col)),
 						    html_cursor_get_position (e->cursor),
-						    position_before), dir);
+						    position_after), dir);
 }
 
 static void
@@ -372,7 +381,7 @@ html_table_delete_column (HTMLTable *t, HTMLEngine *e, gint col, HTMLUndoDirecti
 	HTMLTableCell *cell;
 	HTMLPoint pos;
 	gint r, c;
-	guint position_before;
+	guint position_after;
 
 	/* this command is valid only in table and when this table has > 1 column */
 	if (!t || t->totalCols < 2)
@@ -380,7 +389,6 @@ html_table_delete_column (HTMLTable *t, HTMLEngine *e, gint col, HTMLUndoDirecti
 
 	html_engine_freeze (e);
 
-	position_before = e->cursor->position;
 	column = g_new0 (HTMLTableCell *, t->totalRows);
 
 	backward_before_col (e, t, col);
@@ -410,7 +418,8 @@ html_table_delete_column (HTMLTable *t, HTMLEngine *e, gint col, HTMLUndoDirecti
 	}
 
 	html_cursor_jump_to (e->cursor, e, pos.object, pos.offset);
-	delete_column_setup_undo (e, column, t->totalRows, position_before, col, dir);
+	position_after = e->cursor->position;
+	delete_column_setup_undo (e, column, t->totalRows, position_after, col, dir);
 	t->totalCols --;
 
 	html_object_change_set (HTML_OBJECT (t), HTML_CHANGE_ALL_CALC);
@@ -418,6 +427,33 @@ html_table_delete_column (HTMLTable *t, HTMLEngine *e, gint col, HTMLUndoDirecti
 	html_engine_thaw (e);
 }
 
+static gboolean
+html_engine_get_table_start_end_cells (HTMLEngine *e, HTMLTableCell **start_cell, HTMLTableCell **end_cell)
+{
+	if (!e->cursor->object->parent ||
+	    !e->cursor->object->parent->parent ||
+	    !HTML_IS_TABLE_CELL (e->cursor->object->parent->parent))
+		return FALSE;
+	if (html_engine_is_selection_active (e))        {
+		if (!e->mark->object->parent ||
+		    !e->mark->object->parent->parent ||
+		    !HTML_IS_TABLE_CELL (e->mark->object->parent->parent) ||
+		    HTML_TABLE (e->mark->object->parent->parent->parent) != html_engine_get_table (e))
+			return FALSE;
+		if (html_cursor_precedes (e->cursor, e->mark)) {
+			*start_cell = HTML_TABLE_CELL (e->cursor->object->parent->parent);
+			*end_cell = HTML_TABLE_CELL (e->mark->object->parent->parent);
+		} else {
+			*start_cell = HTML_TABLE_CELL (e->mark->object->parent->parent);
+			*end_cell = HTML_TABLE_CELL (e->cursor->object->parent->parent);
+		}
+	} else {
+		*start_cell = *end_cell = html_engine_get_table_cell (e);
+	}
+
+	return TRUE;
+}
+
 /**
  * html_engine_delete_table_column:
  * @e: An HTML engine.
@@ -428,10 +464,33 @@ html_table_delete_column (HTMLTable *t, HTMLEngine *e, gint col, HTMLUndoDirecti
 void
 html_engine_delete_table_column (HTMLEngine *e)
 {
-	HTMLTableCell *cell = html_engine_get_table_cell (e);
+	HTMLTableCell *start_cell, *end_cell;
+	gint start_col, end_col;
+	HTMLTable *table;
 
-	if (cell)
-		html_table_delete_column (html_engine_get_table (e), e, cell->col, HTML_UNDO_UNDO);
+        table = html_engine_get_table (e);
+        if (!table || !HTML_IS_TABLE (table) ||
+                !html_engine_get_table_start_end_cells (e, &start_cell, &end_cell))       {
+                g_warning ("Invalid table object! Row deletion failed!");
+                return;
+        }
+
+	start_col = start_cell->col;
+	end_col = end_cell->col;
+	html_engine_disable_selection (e);
+	if (start_cell->row == end_cell->row)
+		while (start_col <= end_col--)
+			html_table_delete_column (table, e, start_col, HTML_UNDO_UNDO);
+	else 
+		if ((start_cell->row == end_cell->row - 1) && 
+		   (start_col > end_col + 1)) {
+			while (start_col < table->totalCols)
+				html_table_delete_column (table, e, start_col, HTML_UNDO_UNDO);
+			while (0 <= end_col--)
+				html_table_delete_column (table, e, 0, HTML_UNDO_UNDO);
+		}
+		else
+			html_engine_delete_table (e);
 }
 
 /*
@@ -538,19 +597,24 @@ delete_row_undo_action (HTMLEngine *e, HTMLUndoData *undo_data, HTMLUndoDirectio
 	HTMLTable *table;
 
 	table = html_engine_get_table (e);
+	if (!table) {
+		html_cursor_jump_to_position (e->cursor, e, position_after + 1);
+		table = html_engine_get_table (e);
+	}
+	g_assert (table);
 	g_assert (data->size == table->totalCols);
 	html_table_insert_row (table, e, data->pos, data->cells, html_undo_direction_reverse (dir));
 }
 
 static void
-delete_row_setup_undo (HTMLEngine *e, HTMLTableCell **row_cells, gint size, guint position_before,
+delete_row_setup_undo (HTMLEngine *e, HTMLTableCell **row_cells, gint size, guint position_after,
 		       gint row, HTMLUndoDirection dir)
 {
 	html_undo_add_action (e->undo,
 			      html_undo_action_new ("Delete table row", delete_row_undo_action,
 						    HTML_UNDO_DATA (delete_cells_undo_new (row_cells, size, row)),
 						    html_cursor_get_position (e->cursor),
-						    position_before), dir);
+						    position_after), dir);
 }
 
 static void
@@ -572,7 +636,7 @@ html_table_delete_row (HTMLTable *t, HTMLEngine *e, gint row, HTMLUndoDirection
 	HTMLTableCell *cell;
 	HTMLPoint pos;
 	gint r, c;
-	guint position_before;
+	guint position_after;
 
 	/* this command is valid only in table and when this table has > 1 row */
 	if (!t || t->totalRows < 2)
@@ -580,7 +644,6 @@ html_table_delete_row (HTMLTable *t, HTMLEngine *e, gint row, HTMLUndoDirection
 
 	html_engine_freeze (e);
 
-	position_before = e->cursor->position;
 	row_cells = g_new0 (HTMLTableCell *, t->totalCols);
 
 	backward_before_row (e, t, row);
@@ -611,7 +674,8 @@ html_table_delete_row (HTMLTable *t, HTMLEngine *e, gint row, HTMLUndoDirection
 
 	html_cursor_jump_to (e->cursor, e, pos.object, pos.offset);
 	t->totalRows --;
-	delete_row_setup_undo (e, row_cells, t->totalCols, position_before, row, dir);
+	position_after = e->cursor->position;
+	delete_row_setup_undo (e, row_cells, t->totalCols, position_after, row, dir);
 	html_object_change_set (HTML_OBJECT (t), HTML_CHANGE_ALL_CALC);
 	html_engine_queue_draw (e, HTML_OBJECT (t));
 	html_engine_thaw (e);
@@ -627,10 +691,25 @@ html_table_delete_row (HTMLTable *t, HTMLEngine *e, gint row, HTMLUndoDirection
 void
 html_engine_delete_table_row (HTMLEngine *e)
 {
-	HTMLTableCell *cell = html_engine_get_table_cell (e);
+	HTMLTableCell *start_cell, *end_cell;
+	gint start_row, end_row;
+	HTMLTable *table;
 
-	if (cell)
-		html_table_delete_row (html_engine_get_table (e), e, cell->row, HTML_UNDO_UNDO);
+	table = html_engine_get_table (e);
+	if (!table || !HTML_IS_TABLE (table) ||
+		!html_engine_get_table_start_end_cells (e, &start_cell, &end_cell))	{
+		g_warning ("Invalid table object! Row deletion failed!");
+		return;
+	}
+
+	start_row = start_cell->row;
+	end_row = end_cell->row;
+	html_engine_disable_selection (e);
+	if (end_row - start_row == table->totalRows - 1)
+		html_engine_delete_table (e);
+	else 
+		while (start_row <= end_row--)
+			html_table_delete_row (table, e, start_row, HTML_UNDO_UNDO);
 }
 
 typedef enum {
@@ -1131,6 +1210,8 @@ html_engine_delete_table (HTMLEngine *e)
 {
 	HTMLTable *table;
 
+	html_engine_disable_selection (e);
+
 	table = html_engine_get_table (e);
 
 	if (!table)
