@@ -63,7 +63,6 @@ split (HTMLTextSlave *slave, guint offset, int skip, char *start_pointer)
 	HTMLObject *obj;
 	HTMLObject *new;
 
-	g_return_if_fail (offset >= 0);
 	g_return_if_fail (offset < slave->posLen);
 
 	obj = HTML_OBJECT (slave);
@@ -605,9 +604,7 @@ get_glyph_items_in_range (HTMLTextSlave *slave, HTMLPainter *painter, int start_
 				tmp_gi = pango_glyph_item_split (&glyph_item->glyph_item, slave->owner->text, split_index);
 
 				/* free the beginning we don't need */
-				pango_item_free (tmp_gi->item);
-				pango_glyph_string_free (tmp_gi->glyphs);
-				g_free (tmp_gi);
+				pango_glyph_item_free (tmp_gi);
 				
 			}
 
@@ -627,11 +624,13 @@ get_glyph_items_in_range (HTMLTextSlave *slave, HTMLPainter *painter, int start_
 				tmp_gi2 = pango_glyph_item_split (&tmp_gi1, slave->owner->text, split_index);
 
 				glyph_item->glyph_item = *tmp_gi2;
+				tmp_gi2->item = NULL;
+				tmp_gi2->glyphs = NULL;
 
 				/* free the tmp1 content and tmp2 container, but not the content */
 				pango_item_free (tmp_gi1.item);
 				pango_glyph_string_free (tmp_gi1.glyphs);
-				g_free (tmp_gi2);
+				pango_glyph_item_free (tmp_gi2);
 
 				glyph_item->type = HTML_TEXT_SLAVE_GLYPH_ITEM_CREATED;
 				glyph_item->widths = NULL;
@@ -741,7 +740,8 @@ draw_text (HTMLTextSlave *self,
 	HTMLText *text = self->owner;
 	GSList *cur;
 	int run_width;
-	int selection_start_index, selection_end_index;
+	int selection_start_index = 0;
+	int selection_end_index = 0;
 	int isect_start, isect_end;
 	gboolean selection;
 	GdkColor selection_fg, selection_bg;
@@ -855,7 +855,7 @@ draw_focus_rectangle  (HTMLTextSlave *slave, HTMLPainter *painter, GdkRectangle
 {
 	HTMLGdkPainter *p;
 	GdkGCValues values;
-	gchar dash [2];
+	gint8 dash_list[] = { 1, 1 };
 	HTMLEngine *e;
 
 	if (painter->widget && GTK_IS_HTML (painter->widget))
@@ -873,10 +873,8 @@ draw_focus_rectangle  (HTMLTextSlave *slave, HTMLPainter *painter, GdkRectangle
 									  painter, HTMLTextColor)->color);
 	gdk_gc_get_values (p->gc, &values);
 
-	dash [0] = 1;
-	dash [1] = 1;
 	gdk_gc_set_line_attributes (p->gc, 1, GDK_LINE_ON_OFF_DASH, values.cap_style, values.join_style);
-	gdk_gc_set_dashes (p->gc, 2, dash, 2);
+	gdk_gc_set_dashes (p->gc, 2, dash_list, 2);
 	gdk_draw_rectangle (p->pixmap, p->gc, 0, box->x - p->x1, box->y - p->y1, box->width - 1, box->height - 1);
 	gdk_gc_set_line_attributes (p->gc, 1, values.line_style, values.cap_style, values.join_style);
 }
@@ -913,7 +911,6 @@ draw (HTMLObject *o,
 	HTMLTextSlave *slave;
 	HTMLText *owner;
 	GtkHTMLFontStyle font_style;
-	guint end;
 	GdkRectangle paint;
 
 	/* printf ("slave draw %p\n", o); */
@@ -925,7 +922,6 @@ draw (HTMLObject *o,
 	owner = slave->owner;
 	font_style = html_text_get_font_style (owner);
 
-	end = slave->posStart + slave->posLen;
 	draw_text (slave, p, font_style, x, y, width, height, tx, ty);
 	
 	if (HTML_OBJECT (owner)->draw_focused)
@@ -1110,11 +1106,9 @@ html_text_slave_init (HTMLTextSlave *slave,
 		      guint posStart,
 		      guint posLen)
 {
-	HTMLText *owner_text;
 	HTMLObject *object;
 
 	object = HTML_OBJECT (slave);
-	owner_text = HTML_TEXT (owner);
 
 	html_object_init (object, HTML_OBJECT_CLASS (klass));
 
