@@ -41,6 +41,8 @@ static guint html_tokenizer_signals[HTML_TOKENIZER_LAST_SIGNAL] = { 0 };
 #define TOKEN_BUFFER_SIZE (1 << 10)
 #define INVALID_CHARACTER_MARKER '?'
 
+#define dt(x)
+
 typedef struct _HTMLBlockingToken HTMLBlockingToken;
 typedef struct _HTMLTokenBuffer   HTMLTokenBuffer;
 typedef	enum { Table }            HTMLTokenType;
@@ -87,6 +89,7 @@ struct _HTMLTokenizerPrivate {
 	gboolean select; /* Are we in a <select> block? */
 	gboolean charEntity; /* Are we in an &... sequence? */
 	gboolean extension; /* Are we in an <!-- +GtkHTML: sequence? */
+	gboolean aTag; /* Are we in a <a/> tag*/
  
 	enum {
 		NoneDiscard = 0,
@@ -231,6 +234,7 @@ html_tokenizer_init (HTMLTokenizer *t)
 	p->select = FALSE;
 	p->charEntity = FALSE;
 	p->extension = FALSE;
+	p->aTag = FALSE;
 
 	p->discard = NoneDiscard;
 	p->pending = NonePending;
@@ -257,6 +261,8 @@ html_tokenizer_finalize (GObject *obj)
 
 	g_free (t->priv);
 	t->priv = NULL;
+
+        G_OBJECT_CLASS (parent_class)->finalize (obj);
 }
 
 GtkType
@@ -316,6 +322,8 @@ html_token_buffer_append_token (HTMLTokenBuffer * buf, const gchar *token, gint
 	buf->data [buf->used] = 0;
 	buf->used ++;
 
+	dt(printf ("html_token_buffer_append_token: '%s'\n", buf->data + buf->used - 1 - len));
+
 	return TRUE;
 }
 
@@ -753,6 +761,41 @@ in_script_or_style (HTMLTokenizer *t, const gchar **src)
 	}
 }
 
+static gunichar win1252_to_unicode [32] = {
+	0x20ac,
+	0x81,
+	0x201a,
+	0x0192,
+	0x201e,
+	0x2026,
+	0x2020,
+	0x2021,
+	0x02c6,
+	0x2030,
+	0x0160,
+	0x2039,
+	0x0152,
+	0x8d,
+	0x017d,
+	0x8f,
+	0x90,
+	0x2018,
+	0x2019,
+	0x201c,
+	0x201d,
+	0x2022,
+	0x2013,
+	0x2014,
+	0x02dc,
+	0x2122,
+	0x0161,
+	0x203a,
+	0x0153,
+	0x9d,
+	0x017e,
+	0x0178
+};
+
 static void
 add_unichar (HTMLTokenizer *t, gunichar wc)
 {
@@ -760,6 +803,15 @@ add_unichar (HTMLTokenizer *t, gunichar wc)
 
 	p->utf8_length = 0;
 
+	/*
+	  chars in range 128 - 159 are control characters in unicode,
+	  but most browsers treat them as windows 1252
+	  encoded characters and translate them to unicode
+	  it's broken, but we do the same here
+	*/
+	if (wc > 127 && wc < 160)
+		wc = win1252_to_unicode [wc - 128];
+
 	if (wc != '\0') {
 		p->dest += g_unichar_to_utf8 (wc, p->dest);
 		*(p->dest) = 0;
@@ -906,7 +958,7 @@ in_tag (HTMLTokenizer *t, const gchar **src)
 
 	p->startTag = FALSE;
 	if (**src == '/') {
-		if (p->pending == LFPending) {
+		if (p->pending == LFPending  && !p->pre) {
 			p->pending = NonePending;
 		}
 	}
@@ -1043,6 +1095,9 @@ end_tag (HTMLTokenizer *t, const gchar **src)
 	}
 	else if (strncmp (p->buffer + 2, "tablesdkl", 9) == 0) {
 		html_tokenizer_blocking_push (t, Table);
+	}	
+	else if (strncmp (p->buffer + 2, "/a", 2) == 0) {
+		p->aTag = FALSE;
 	}
 	else {
 		if (p->blocking) {
@@ -1100,6 +1155,8 @@ in_crlf (HTMLTokenizer *t, const gchar **src)
 static void
 in_space_or_tab (HTMLTokenizer *t, const gchar **src)
 {
+	gchar *ptr;
+	
 	if (t->priv->tquote) {
 		if (t->priv->discard == NoneDiscard)
 			t->priv->pending = SpacePending;
@@ -1108,6 +1165,9 @@ in_space_or_tab (HTMLTokenizer *t, const gchar **src)
 		t->priv->searchCount = 0; /* Stop looking for <!-- sequence */
 		if (t->priv->discard == NoneDiscard)
 			t->priv->pending = SpacePending;
+		ptr = t->priv->buffer;
+		if (ptr[1] == '<' && ptr[2] == 'a' && strlen (ptr) == 3)
+			t->priv->aTag = TRUE;
 	}
 	else if (t->priv->pre || t->priv->textarea) {
 		if (t->priv->pending)
@@ -1237,7 +1297,7 @@ html_tokenizer_tokenize_one_char (HTMLTokenizer *t, const gchar **src)
 		in_entity (t, src);
 	else if (p->startTag)
 		in_tag (t, src);
-	else if (**src == '&')
+	else if (**src == '&' && !p->aTag)
 		start_entity (t, src);
 	else if (**src == '<' && !p->tag)
 		start_tag (t, src);
